read.o read.d : ../../../icu/source/tools/genrb/read.c \
 ../../../icu/source/tools/genrb/read.h \
 ../../../icu/source/common/unicode/utypes.h \
 ../../../icu/source/common/unicode/umachine.h \
 ../../../icu/source/common/unicode/ptypes.h \
 ../../../icu/source/common/unicode/platform.h \
 ../../../icu/source/common/unicode/uconfig.h \
 ../../../icu/source/common/unicode/uvernum.h \
 ../../../icu/source/common/unicode/urename.h \
 ../../../icu/source/common/unicode/uversion.h \
 ../../../icu/source/common/unicode/utf.h \
 ../../../icu/source/common/unicode/utf8.h \
 ../../../icu/source/common/unicode/utf16.h \
 ../../../icu/source/common/unicode/utf_old.h \
 ../../../icu/source/tools/genrb/ustr.h \
 ../../../icu/source/tools/genrb/../toolutil/ucbuf.h \
 ../../../icu/source/common/unicode/localpointer.h \
 ../../../icu/source/common/unicode/ucnv.h \
 ../../../icu/source/common/unicode/ucnv_err.h \
 ../../../icu/source/common/unicode/uenum.h \
 ../../../icu/source/tools/genrb/../toolutil/filestrm.h \
 ../../../icu/source/tools/genrb/errmsg.h \
 ../../../icu/source/tools/genrb/../toolutil/toolutil.h \
 ../../../icu/source/common/unicode/ustring.h \
 ../../../icu/source/common/unicode/putil.h \
 ../../../icu/source/common/unicode/uiter.h
