gennorm2.o gennorm2.d : ../../../icu/source/tools/gennorm2/gennorm2.cpp \
 ../../../icu/source/common/unicode/utypes.h \
 ../../../icu/source/common/unicode/umachine.h \
 ../../../icu/source/common/unicode/ptypes.h \
 ../../../icu/source/common/unicode/platform.h \
 ../../../icu/source/common/unicode/uconfig.h \
 ../../../icu/source/common/unicode/uvernum.h \
 ../../../icu/source/common/unicode/urename.h \
 ../../../icu/source/common/unicode/uversion.h \
 ../../../icu/source/common/unicode/utf.h \
 ../../../icu/source/common/unicode/utf8.h \
 ../../../icu/source/common/unicode/utf16.h \
 ../../../icu/source/common/unicode/utf_old.h \
 ../../../icu/source/tools/gennorm2/n2builder.h \
 ../../../icu/source/common/unicode/errorcode.h \
 ../../../icu/source/common/unicode/uobject.h \
 ../../../icu/source/common/unicode/umutablecptrie.h \
 ../../../icu/source/common/unicode/ucpmap.h \
 ../../../icu/source/common/unicode/ucptrie.h \
 ../../../icu/source/common/unicode/localpointer.h \
 ../../../icu/source/common/unicode/unistr.h \
 ../../../icu/source/common/unicode/char16ptr.h \
 ../../../icu/source/common/unicode/rep.h \
 ../../../icu/source/common/unicode/std_string.h \
 ../../../icu/source/common/unicode/stringpiece.h \
 ../../../icu/source/common/unicode/bytestream.h \
 ../../../icu/source/common/normalizer2impl.h \
 ../../../icu/source/common/unicode/utypes.h \
 ../../../icu/source/common/unicode/normalizer2.h \
 ../../../icu/source/common/unicode/uniset.h \
 ../../../icu/source/common/unicode/unifilt.h \
 ../../../icu/source/common/unicode/unifunct.h \
 ../../../icu/source/common/unicode/unimatch.h \
 ../../../icu/source/common/unicode/uset.h \
 ../../../icu/source/common/unicode/uchar.h \
 ../../../icu/source/common/unicode/stringoptions.h \
 ../../../icu/source/common/unicode/unorm2.h \
 ../../../icu/source/common/unicode/ucptrie.h \
 ../../../icu/source/common/unicode/unistr.h \
 ../../../icu/source/common/unicode/unorm.h \
 ../../../icu/source/common/unicode/uiter.h \
 ../../../icu/source/common/unicode/utf.h \
 ../../../icu/source/common/unicode/utf16.h \
 ../../../icu/source/common/mutex.h \
 ../../../icu/source/common/unicode/uobject.h \
 ../../../icu/source/common/umutex.h \
 ../../../icu/source/common/unicode/uclean.h \
 ../../../icu/source/common/putilimp.h \
 ../../../icu/source/common/unicode/putil.h \
 ../../../icu/source/common/udataswp.h \
 ../../../icu/source/common/uset_imp.h \
 ../../../icu/source/common/unicode/uset.h \
 ../../../icu/source/tools/gennorm2/../toolutil/toolutil.h \
 ../../../icu/source/tools/gennorm2/norms.h \
 ../../../icu/source/common/uvectr32.h ../../../icu/source/common/uhash.h \
 ../../../icu/source/common/cmemory.h \
 ../../../icu/source/common/unicode/localpointer.h \
 ../../../icu/source/common/uassert.h \
 ../../../icu/source/common/uelement.h \
 ../../../icu/source/common/unicode/putil.h \
 ../../../icu/source/common/charstr.h \
 ../../../icu/source/tools/gennorm2/../toolutil/uoptions.h \
 ../../../icu/source/tools/gennorm2/../toolutil/uparse.h
