GENCCODE_ASSEMBLY_TYPE=-a gcc
SO=so
SOBJ=so
A=a
LIBPREFIX=lib
LIB_EXT_ORDER=.71.1
COMPILE=gcc  -isystemFIXMESTAGINGDIRHOST/usr/include -D_REENTRANT  -DU_HAVE_ELF_H=1 -DU_HAVE_STRTOD_L=1 -DU_HAVE_XLOCALE_H=0  -DU_ATTRIBUTE_DEPRECATED= -isystemFIXMESTAGINGDIRHOST/usr/include -O2 -pipe -std=c11 -Wall -pedantic -Wshadow -Wpointer-arith -Wmissing-prototypes -Wwrite-strings   -c
LIBFLAGS=-IFIXMESTAGINGDIRHOST/usr/include -DPIC -fPIC
GENLIB=gcc  -isystemFIXMESTAGINGDIRHOST/usr/include -O2 -pipe -std=c11 -Wall -pedantic -Wshadow -Wpointer-arith -Wmissing-prototypes -Wwrite-strings   -LFIXMESTAGINGDIRHOST/usr/lib                         -LFIXMESTAGINGDIRHOST/lib                         -Wl,--enable-new-dtags                         -Wl,-rpath-link,FIXMESTAGINGDIRHOST/usr/lib                         -Wl,-rpath-link,FIXMESTAGINGDIRHOST/lib                         -Wl,-rpath,FIXMESTAGINGDIRHOST/usr/lib                         -Wl,-rpath,FIXMESTAGINGDIRHOST/lib                         -Wl,-O1 -Wl,--allow-shlib-undefined -Wl,--dynamic-linker=/scratch/jenkins-BUILDS-eSDK-2023.1_stable-pipeline-build-136_MbMhDivFull/build/tmp/sysroots-uninative/x86_64-linux/lib/ld-linux-x86-64.so.2  -shared -Wl,-Bsymbolic
LDICUDTFLAGS=-nodefaultlibs -nostdlib
LD_SONAME=-Wl,-soname -Wl,
RPATH_FLAGS=
BIR_LDFLAGS=-Wl,-Bsymbolic
AR=ar
ARFLAGS=r
RANLIB=ranlib -D
INSTALL_CMD=FIXME_HOSTTOOLS_DIR/install -c
